import string
import tempfile
from cStringIO import StringIO
from Acquisition import aq_base, aq_inner, aq_parent, Implicit

from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.PortalFolder  import PortalFolder
from Products.CMFCore.PortalFolder  import PortalFolder
from Products.CMFCore.Expression import Expression

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import addDirectoryViews, DirectoryViewSurrogate, DirectoryView
from Products.DCWorkflow.States import StateDefinition
from Products.CMFStaging.LockTool import LockObjects, UnlockObjects
from Products.CMFStaging.VersionsTool import UseVersionControl
from Products.ZopeVersionControl.Utility import VersionControlError

from Products.PloneVersionControl.LockTool import LockTool
from Products.PloneVersionControl.VersionsTool import VersionsTool
from Products.PloneVersionControl.WorkflowWithRepositoryTool import WorkflowWithRepositoryTool
from Products.PloneVersionControl import ploneversioncontrol_globals
from Products.PloneVersionControl.utils import log

# for setting the properties, workflow, and versioning of content types
VC_CONTENT = ['File','Document']
VC_FOLDER  = []
VC_TYPES   = VC_CONTENT + VC_FOLDER
VC_METATYPES   = ['Portal File','Document']
DEFAULT_FOLDER  = ['Plone Folder', 'Large Plone Folder']

def setupWorkflow(self, out):
    try:
        self._delObject('portal_workflow')
    except AttributeError:
        pass

    self.manage_addProduct['PloneVersionControl'].manage_addTool(
            WorkflowWithRepositoryTool.meta_type)
    out.write('Added portal_workflow.\n')

    workflows=({'id':'plone_workflow',
                'type':'plone_workflow '+\
                '(Default Workflow [Plone])',
                'meta_types':[] },
               {'id':'folder_workflow',
                'type':'folder_workflow '+\
                '(Folder Workflow [Plone])',
                'meta_types':DEFAULT_FOLDER },
               {'id':'vc_default_workflow',
                'type':'plone_workflow_versioning '+ \
                '(Default Workflow with Versioning [Plone])',
                'meta_types':VC_CONTENT },
               {'id':'vc_folder_workflow',
                'type':'plone_folder_workflow_versioning '+ \
                '(Default Folder Workflow with Versioning [Plone])',
                'meta_types':VC_FOLDER },
               )

    wf_tool=getToolByName(self, 'portal_workflow')
    for wf in workflows:
        id = wf['id']
        type = wf['type']
        meta_types = wf['meta_types']
        if id in wf_tool.objectIds():
            wf_tool._delObjects(id)
        wf_tool.manage_addWorkflow(id=id, workflow_type=type)
        wf_tool.setChainForPortalTypes(pt_names = meta_types, chain = id)
        tipos = ", ".join(meta_types)
        out.write('Workflow %s set for type(s) %s.\n' % (id, tipos))

    wf_tool.setDefaultChain( 'plone_workflow' )
    out.write('Workflow plone_workflow set as default chain.\n')

def setupSlots(self, out):
    left_slots = getattr(self, 'left_slots')
    left_slots = left_slots + ('here/version_slot/macros/portlet',)
    self._updateProperty('left_slots', left_slots)
    out.write('Added Versioning Slot to Portal.\n')

def setupSkin(self, out, skinFolder):
    skinstool=getToolByName(self, 'portal_skins')

    addDirectoryViews(skinstool, 'skins', ploneversioncontrol_globals)

    for skin in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skin)
        path = map( string.strip, string.split( path,',' ) )

        if not skinFolder in path:
            try:
                path.insert( path.index( 'custom')+1, skinFolder )
            except ValueError:
                path.append(skinFolder)
            path = string.join( path, ', ' )
            skinstool.addSkinSelection( skin, path )
            out.write('Subskin sucessfully installed into skin %s.\n' % skin)
        else:
            out.write('Subskin was already installed into skin %s.\n' % skin)

def applyVersionControl(self, recursive=0):
    if hasattr(aq_base(self), 'haveRepository') and \
    self.haveRepository() and \
    self.isAVersionableResource(self) and \
    self.meta_type in VC_METATYPES and \
    not self.isUnderVersionControl():
        self.applyVersionControl()
    
    if recursive and hasattr(aq_base(self), 'objectValues'):
        for obj in self.objectValues():
            applyVersionControl(obj, recursive)


def applyWorkflow(self, out, recursive=0):
    out.write('    applywf start: %s \n' % self.id)
    if isinstance(self, PortalFolder) or isinstance(self, PortalContent):
        out.write('    applywf notifycreated: %s \n' % self.id)
        self.portal_workflow.notifyCreated(self)
    else:
        out.write('    applywf SKIP: %s \n' % self.id)

    if recursive and isinstance(self, PortalFolder):
        out.write('    applywf recurse on folder: %s \n' % self.id)
        for obj in self.listFolderContents():
            applyWorkflow(obj, out, recursive)


def setPermissions(self, perms_list, roles_list=('Manager', 'Owner')):
    """
    setPermissions(self) => Set standard permissions / roles
    """
    # As a default behavior, newly-created permissions are granted to owner and manager.
    # To change this, just comment this code and grab back the code commented below to
    # make it suit your needs.        
    for perm in perms_list:
        self.manage_permission(
            perm,
            roles_list,
            acquire = 1
            )


def setupProperties(self, out):
    out.write('Edit form properties to check locking... \n')
    # change the edit form properties to check locking
    fp=self.portal_properties.form_properties
    props={}
    for id, prop in fp.propertyItems():
        if id.endswith('edit_form') and prop.startswith('validate'):
            props[id]='validate_locking,'+prop
    fp.manage_changeProperties(props)

    out.write('Add new project properties... \n')
    # install project properties
    ptool = getToolByName(self, 'portal_properties')
    psheet = getattr(ptool, 'project_properties', None)

    if not psheet:
        ptool.addPropertySheet('project_properties', 'Properties for the project')
        ps = getattr(ptool, 'project_properties')

        ps._properties = ps._properties + (
            {'id':'version_control_types', 'type':'lines', 'mode':'w'},
            )
        ps._updateProperty('version_control_types', VC_METATYPES)


def setupActions(self, out):
    # And now update the relevant portal_type actions
    out.write("Inspecting portal_types")
    tool = getToolByName(self, 'portal_types')
    for ptype in tool.objectValues():
        if ptype.getId() in VC_TYPES:
            #add the action for viewing versioning
            action = ptype.getActionById( 'versions', default=None )
            if action is None:
                out.write( 'Added Revisions tab for %s' % ptype.getId() )
                ptype.addAction( 'versions'
                               , 'Versions'
                               , 'string:${object_url}/object_version_control_form'
                               , ''
                               , 'Modify portal content'
                               , 'object'
                               , visible=1
                               )

## Default Layout: the install script
## Version Control: 1 site w/ version control and locking
def installVersionControl(self):
    
    """
    create a single versioned site.
    """
    out = StringIO()

    portal = aq_parent(aq_inner(self))
    out.write('Got a reference to parent.\n')
    
    zvc = portal.manage_addProduct['ZopeVersionControl']
    zvc.addRepository('VersionRepository')
    out.write('Added VersionRepository.\n')

    self.manage_addProduct['PloneVersionControl'].manage_addTool(LockTool.meta_type)
    out.write('Added portal_lock.\n')

    self.manage_addProduct['PloneVersionControl'].manage_addTool(VersionsTool.meta_type)
    out.write('Added portal_versions.\n')

    self.manage_addProduct['CMFStaging'].manage_addWorkflowRepository('workflow_repository')
    out.write('Added workflow_repository.\n')

    out.write('Call setupWorkflow... \n')
    setupWorkflow(self, out)

    out.write('Call setupSlots... \n')
    setupSlots(self, out)

    out.write('Call setupSkin... \n')
    setupSkin(self, out, 'plone_version_control')

    out.write('Call setupActions... \n')
    setupActions(self, out)

    out.write('Call setupProperties... \n')
    setupProperties(self, out)

    out.write('Call setPermissions... \n')
    setPermissions( self, ['Use version control'], ['Anonymous','Authenticated','Manager','Member','Owner','Reviewer'] )

    out.write('Call notifycreated for each item... \n')
    applyWorkflow(self,out,1)

    out.write('Call applyVersionControl... \n')
    applyVersionControl(self, 1)

    out.write('Completed PloneVersionControl Configuration. \n')
    return out.getvalue()

def install(self):
    return installVersionControl(self)


