from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.DCWorkflow.Transitions import TRIGGER_AUTOMATIC
from Products.CMFCore.CMFCorePermissions import RequestReview, \
                                                ModifyPortalContent, \
                                                ReviewPortalContent
from AccessControl.Permissions import view, access_contents_information
from Products.PythonScripts.PythonScript import PythonScript
from Products.CMFPlone.PloneWorkflow import setupDefaultPloneWorkflow
from Products.CMFPlone.FolderWorkflow import setupFolderWorkflow
from Products.CMFStaging.LockTool import LockObjects, UnlockObjects

p_modify = ModifyPortalContent
p_review = ReviewPortalContent
p_lock = LockObjects
p_unlock = UnlockObjects

r_manager = 'Manager'
r_owner = 'Owner'
r_reviewer = 'Reviewer'

def setAfterScript(wf, tdef_id, script_id):
    tdef=wf.transitions[tdef_id]
    scripts = wf.scripts

    if not script_id in scripts.objectIds():
        scripts._setObject(script_id, PythonScript(script_id))
        getattr(scripts, script_id).write(transitionScripts[script_id])

    tdef.after_script_name = script_id
    
def setBeforeScript(wf, tdef_id, script_id):
    tdef=wf.transitions[tdef_id]
    scripts = wf.scripts

    if not script_id in scripts.objectIds():
        scripts._setObject(script_id, PythonScript(script_id))
        getattr(scripts, script_id).write(transitionScripts[script_id])

    tdef.script_name = script_id

def setupPloneWorkflowWithVersioning(wf):
    """
    sets up a workflow on an enviroment with Versioning
    """

    wf.states.addState('created')
    sdef = wf.states['created']
    sdef.setProperties(
        title='Automatic transition to setup permissions and lock',
        transitions=('initialize',))
    #sdef.setPermission(p_modify, 0, ())

    wf.states.setInitialState('created')

    transitions = ('checkin', 'checkout')
    for t in transitions:
        wf.transitions.addTransition(t)

    wf.transitions.addTransition('initialize')
    tdef = wf.transitions['initialize']
    tdef.setProperties(
        title='Initialize object and unlock',
        trigger_type=0, 
        new_state_id='visible',
        actbox_name='Initialize object and unlock',
        actbox_url='',
        props={'guard_permissions':p_review})

    tdef = wf.transitions['checkout']
    tdef.setProperties(
        title='Checkout',
        new_state_id='',
        actbox_name='Checkout',
        actbox_url='',
        props={'guard_permissions':p_review})

    tdef = wf.transitions['checkin']
    tdef.setProperties(
        title='Checkin',
        new_state_id='',
        actbox_name='Checkin',
        actbox_url='',
        props={'guard_permissions':p_review})


    setAfterScript(wf, 'initialize', 'checkout')
    setAfterScript(wf, 'checkout', 'checkout')
    setAfterScript(wf, 'checkin', 'checkin')

    for t in wf.transitions.objectIds():
        if t not in transitions:
            tdef = wf.transitions[t]
            g = tdef.getGuard()
            g.changeFromProperties({'guard_permissions':p_modify})
            tdef.guard = g

    for s in ('visible', 'private', 'published'):
        sdef = wf.states[s]
        sdef.setProperties(
            title=sdef.title,
            transitions=sdef.getTransitions() + transitions)



def createPloneWorkflowWithVersioning(id):
    ob=DCWorkflowDefinition(id)
    setupDefaultPloneWorkflow(ob)
    setupPloneWorkflowWithVersioning(ob)
    return ob

addWorkflowFactory( createPloneWorkflowWithVersioning
                    , id='plone_workflow_versioning'
                    , title='Default Workflow with Versioning [Plone]')	   


def createPloneFolderWorkflowWithVersioning(id):
    ob=DCWorkflowDefinition(id)
    setupFolderWorkflow(ob)
    setupPloneWorkflowWithVersioning(ob)
    return ob

addWorkflowFactory( createPloneFolderWorkflowWithVersioning \
                    , id='plone_folder_workflow_versioning'
                    , title='Default Folder Workflow with Versioning [Plone]')

transitionScripts={'checkin':"""
## Script (Python) "checkin"
##parameters=sci
# Check in the object to a repository, disallowing changes.
#
object = sci.object
message = sci.kwargs.has_key('comment') and sci.kwargs['comment'] or ''
vt = object.portal_versions
if vt.isCheckedOut(object):
  vt.checkin(object, message)

pl = object.portal_lock
pl.unlock(object, message)
object.manage_permission('Modify portal content', ('Manager'), 0)

""",
'checkout':"""
## Script (Python) "checkout"
##parameters=sci
# Check out the object from a repository, allowing changes.
#
# For workflows that control Versioning, it makes sense to call this script
# before all transitions.
object = sci.object
vt = object.portal_versions
if not vt.isCheckedOut(object):
  vt.checkout(object)

pl = object.portal_lock
pl.lock(object)
object.manage_permission('Modify portal content', ('Manager', 'Owner', 'Reviewer'), 0)

return 1
""",
}
